package de.hsrg.gui;

import java.awt.image.BufferedImage;
import java.io.IOException;
import java.awt.Graphics;
import java.awt.Color;
import java.awt.Point;
import java.io.File;
import java.util.concurrent.locks.Condition;
import java.util.concurrent.locks.Lock;
import java.util.concurrent.locks.ReentrantLock;

import javax.media.jai.ParameterBlockJAI;
import javax.media.jai.RenderedOp;
import javax.media.jai.TiledImage;
import javax.imageio.ImageIO;
import javax.media.jai.JAI;
import javax.swing.JPanel;

public class BlueCube extends JPanel implements Runnable {		
	private static final long serialVersionUID = 7858483147493385733L;
	public final static int DEFAULT_ANGLE = -22, DEFAULT_DIRECTION = 1, 
			                DEFAULT_VELOCITY = 8, MAX_VELOCITY = 16;
	final static double WHITE_RGB[] = {255.0,255.0,255.0};
	final static String SOURCE_FILE = "./cube.jpg";
	final static double CONVERT = Math.PI/ 180;
	private int width, height, inset, angle, velocity, direction,
	            cubeOrigin, cubeX, cubeY, cubeSize;	
	private ParameterBlockJAI cubeSettings;
	private BufferedImage background, cube;
	private volatile boolean suspended;	
	private Condition running;
	private Lock paused;

	public BlueCube(int width, int height)  {
		angle = DEFAULT_ANGLE;  velocity = DEFAULT_VELOCITY;  direction = DEFAULT_DIRECTION;		
		paused = new ReentrantLock( );  running = paused.newCondition( );
		this.width = width;  this.height = height;		
		try {
			background = ImageIO.read(new File(SOURCE_FILE));
		} catch (IOException e) {  }
		cubeX = width / 9; cubeY = height / 5; 
		cubeOrigin = width / 18; cubeSize = (int)(height / 1.6);
		TiledImage source = new TiledImage(background, width, height);
		TiledImage cubeData = // extract cube data for rotation
				source.getSubImage(cubeOrigin, cubeOrigin, cubeSize, cubeSize);
	    cubeSettings = new ParameterBlockJAI("rotate");	
		cubeSettings.setParameter("backgroundValues", WHITE_RGB);
		cubeSettings.addSource(cubeData);
		setDoubleBuffered(true);
	}
	
	public void paintComponent(Graphics handle)  {
		Point center = new Point((getWidth( ) - width) / 2,
				                 (getHeight( ) - height) / 2); 
		handle.drawImage( background, center.x, center.y,
				          width, height, this);
		update(handle);
	}
	
	public void update(Graphics handle)  {
		cubeSettings.setParameter("angle", new Float(angle * CONVERT)); 
		RenderedOp op = JAI.create("rotate", cubeSettings);
		cube = op.getRendering( ).getAsBufferedImage( );	
		cubeX = (getWidth( ) - width) / 2;  cubeY = (getHeight( ) - height) / 2; 
		inset = width / 7;		
		handle.setColor(Color.WHITE);  // fade out
		handle.fillRect( cubeX, cubeY, // fixed cube
				         (int)(cubeSize * 1.5), (int)(cubeSize * 1.5) );
		handle.drawImage(cube, cubeX + inset, cubeY + inset, cubeSize, cubeSize, this);
	}	
	
	public void run( )  {
		for ( ; ; )  {
			while (suspended)  {
				paused.lock( );
				running.awaitUninterruptibly( );
				paused.unlock( );
			}
			rotate(angle = angle + direction % 360);
			try {
				Thread.sleep(20 - velocity);
			}  catch (InterruptedException e) {  }
		}
	}

	public void setVelocity(int velocity) {
		if (velocity >= DEFAULT_VELOCITY)  {
			this.direction = DEFAULT_DIRECTION;
			this.velocity = velocity;
		}  else  {
			this.direction = -DEFAULT_DIRECTION;
			this.velocity = MAX_VELOCITY - velocity;
		}	
	}
	
	public void rotate(int angle)  {
		this.angle = angle;
		repaint( );
	}	
	
	public void stopRotation( )  {
		paused.lock( );
		suspended = true;
		paused.unlock( );		
	}
	
	public void resumeRotation( )  {
		paused.lock( );
		suspended = false;
		running.signal( );
		paused.unlock( );		
	}
	
	public void reset( )  {
		rotate(DEFAULT_ANGLE);
	}	
	
	public BufferedImage snapshot(int angle)  {
		BufferedImage imageBuffer = 
		 new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics handle = imageBuffer.getGraphics( );
        handle.drawImage( background, 0, 0,
                          width, height, this );
        rotate(angle);
		cubeSettings.setParameter("angle", new Float(angle * CONVERT)); 
		RenderedOp op = JAI.create("rotate", cubeSettings);
		cube = op.getRendering( ).getAsBufferedImage( );	
		handle.setColor(Color.WHITE);  // fade out
		handle.fillRect( 0, 0, // fixed cube
				         (int)(cubeSize * 1.5), (int)(cubeSize * 1.5) );
		handle.drawImage(cube, cubeX, cubeY, cubeSize, cubeSize, this);
        return imageBuffer;
}

	
}
